{EventEmitter} = require "events"
fs = require "fs"
vm = require "vm"

coffee = require "coffee-script"
find = require "findit"
jsdom = require "jsdom"

util = require "./util"
paths = require "./paths"

class Environment extends EventEmitter

    constructor: ({ jasmine, includes, html } = {}) ->
        @jasmine = jasmine ? paths.jasmine.default
        @includes = includes ? []
        @specs = []
        @html = html ? "<html><head></head><body></body></html>"

    use: (middleware) ->
        middleware.call(this, this)
        this

    add: (path) ->
        return unless path?
        if util.type(path) is 'array'
            @add(p) for p in path
        else
            stat = fs.statSync path
            if stat.isDirectory()
                @directory.apply this, arguments
            else
                @file.apply this, arguments
        this

    file: (path) ->
        if util.type(path) is 'array'
            @file(p) for p in path
        else
            for p in arguments
                throw new Error() unless fs.statSync(p).isFile()
                @specs.push(path) unless path in @specs
        this

    directory: (dir, criteria) ->
        if util.type(dir) is 'array'
            @directory(d, criteria) for d in dir
        else
            criteria = criteria ? /\.(node|js|coffee)$/i
            matcher = criteria
            if util.type(matcher) is 'regexp'
                matcher = (path) -> path.match(criteria)?
            if util.type(matcher) is not 'function'
                matcher = (path) -> path is criteria
            @file(file for file in find.sync(dir) when matcher(file))
        this

    report: (event, data) =>
        @emit event, data, this

    load: (opts, callback) ->
        if util.type(opts) is 'function'
            callback = opts
            opts = undefined

        opts = {} unless opts?
        includes = @includes.concat(opts.includes ? [])
        includes = [@jasmine].concat(includes)
        html = opts.html ? @html

        jsdom.env html, includes, (err, window) =>
            throw err if err
            @window = window
            @env = @window.jasmine.getEnv()
            @env.addReporter util.reporter(@report)
            callback(window) if util.type(callback) is 'function'
        this

    compile: (context) ->
        context = context ? @window
        it = context.it
        describe = context.describe
        for spec in @specs
            code = util.read(spec)
            meta = file: spec, source: code
            context.it = -> it.apply(this, arguments).meta = meta
            context.describe = -> describe.apply(this, arguments).meta = meta

            if spec.lastIndexOf(".coffee") is (spec.length - 7)
                coffee.eval(code, {
                    sandbox: @window
                    filename: spec
                })
            else
                vm.runInNewContext code, @window, spec
        context.it = it
        context.describe = describe

    execute: ->
        return @load(=> @execute()) unless @window
        @compile()
        @env.execute()
        this

Environment.create = create = (opts) ->
    return new Environment(opts)

module.exports = Environment
