ANSIColors =
  green: '\033[32m' # Green
  red: '\033[31m' # Red
  normal: '\033[0m' # Normal

NoColors =
  green: ''
  red: ''
  normal: ''

class TerminalReporter
    constructor: (env, { verbose, color } = {}) ->
        @env = env
        color = color ? true
        @is_verbose = verbose || false
        @colors = if color is not false then ANSIColors else NoColors
        @counter = 0
        @failed = []
        @start_time = 0

    write: ->
        process.stdout.write.apply process.stdout, arguments

    print: ->
        console.log.apply console, arguments

    print_failed: (failed) ->
        indent = "  "
        lines = (suite.description for suite in @lineage(failed.suite))
        @print ""
        @write @color("X ", "red")
        @print "#{ failed.spec.items_[0].message }"
        @print "#{ indent }Suite:  #{ failed.suite.description }"
        @print "#{ indent }Spec:   #{ failed.spec.description }"
        @print "#{ indent }Source: #{ failed.suite.meta.file }"
        @print ""
        for entry in failed.trace
            @print "#{ indent }#{ entry.file }:#{ entry.line }:#{ entry.column }"

    lineage: (suite) ->
        group = suite
        lineage = []
        while(group)
            lineage.unshift group
            group = group.parentSuite
        lineage

    trace: (text, file, env) ->
        text = text.replace(/\[object Context\]/g, file) if file
        for line in text.split(/\n/)
            continue unless line.indexOf(env.jasmine) is -1
            trace = line.match(/\(([^\)]*)\)$/)
            continue unless trace?
            line = trace[1]
            line = trace[1].split(':')
            file: line[0], line: line[1], column: line[2]

    color: (str, color) ->
        s = @colors[color ? 'normal'] + str + @colors.normal

    summary: (runner) ->
        info = (count, word) ->
            "#{ count } #{ (if count is 1 then word else "#{ word }s") }"

        results = runner.results()
        suites = runner.suites()
        msg = [
            info(suites.length, 'test')
            info(results.totalCount, 'assertion')
            info(results.failedCount, 'failure')
        ]
        msg.join(', ')

    start: (runner, env) =>
        @start_time = Number(new Date)

    suite: (suite, env) =>
        specResults = suite.results()

        for spec in specResults.items_
            if spec.failedCount > 0 and spec.description
                for result in spec.items_
                    continue if result.passed_
                    trace = @trace result.trace.stack, suite.meta.file, env
                    @failed.push {
                        spec: spec,
                        suite: suite,
                        trace: trace
                    }

    spec: (spec, env) =>
        @write @color('.', 'green') if spec.results().passed()
        @write @color('F', 'red') unless spec.results().passed()
        return if @counter++ < 50
        @counter = 0
        @write '\n'

    finish: (runner, env) =>
        elapsed = (Number(new Date) - @start_time) / 1000
        color = if @failed.length > 0 then 'red' else 'green'
        @print ""
        @print_failed entry for entry in @failed
        @print ""
        @print @color("Success!", color) if color is 'green'
        @print @color("#{ @summary(runner)}, #{ elapsed } seconds.", color)


module.exports = terminal = (opts = {}) ->
    term = new TerminalReporter(opts)
    (env) ->
        env.on 'start', term.start
        env.on 'spec.end', term.spec
        env.on 'suite.end', term.suite
        env.on 'end', term.finish
